/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * END COPYRIGHT BLOCK **/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "i18n.h"

#include "getstrmem.h"

#include "libadminutil/resource.h"
#include "coreres.h"

Resource *hResource = NULL;
char empty_string[] = "";

char*
XP_GetStringFromMemory(char* strLibraryName,int iToken);



void
XP_InitStringDatabase(char* pathCWD, char* databaseName)
{
    hResource = core_res_init_resource (pathCWD, databaseName);
}

char *XP_GetPropertyString(char* strLibraryName,int iToken, ACCEPT_LANGUAGE_LIST lang)
{
    char *key_name;
    char *result = NULL;

    if (hResource == NULL)
        return NULL;

	/*creating the key*/
	key_name=(char*)malloc(strlen(strLibraryName) + 10);
	sprintf(key_name, "%s-%d", strLibraryName, iToken);
	if(key_name == NULL)
		return NULL;

    result = (char *) core_res_getstring(hResource, key_name, lang) ;

    if (key_name)
        free (key_name);

    if (result == NULL)
        return empty_string;
    else
        return result ;
}

char*
XP_GetStringFromDatabase(char* strLibraryName,
                         char* strLanguage,
                         int key)
{
    char *result = NULL;
    ACCEPT_LANGUAGE_LIST alanglist;
	int n;

    /*
     * display first choice language if available, otherwise 
     * use default which is english in most case
     */
    if (hResource) { 
        n = XP_AccLangList (strLanguage, alanglist);
		if (n >= MAX_ACCEPT_LANGUAGE)
			alanglist[MAX_ACCEPT_LANGUAGE-1][0] = '\0';
		else
			alanglist[n][0] = '\0';
		result = XP_GetPropertyString(strLibraryName, key, alanglist);
    }

    /* we should never come here. */
    if (result == NULL)
        result = XP_GetStringFromMemory(strLibraryName,key);
    return result;
}


char*
XP_GetStringFromMemory(char* strLibraryName,int iToken)
{
  /*
   * In memory model called by XP_GetStringFromDatabase
   * does not use database (nsres, et al.).
   *
   * This function uses hash table for library lookup
   * and direct lookup for string.
   *
   * This function is thread safe.
   */


  unsigned hashKey;
  int      found = 0;
  unsigned uToken = iToken;
  char*    cPtr;
  DATABIN* pBucket;

  /* calculate hash key */
  hashKey = 0;
  cPtr = strLibraryName;
  while (*cPtr) {
	hashKey += *(cPtr++);
  }
  hashKey &= BUCKET_MASK;

  /* get bucket for this hash key */
  pBucket = buckets[hashKey];

  /* search overflow buckets */
  while (*(pBucket->pLibraryName)!='\0') {
    if (strcmp(pBucket->pLibraryName,strLibraryName)==0) {
	  found = 1;
	  break;
    }
    pBucket++;
  }

  if (!found) {
    return emptyString;
  }

  if (uToken<=pBucket->numberOfStringsInLibrary) {
      return pBucket->pArrayOfLibraryStrings[uToken];
    } else {
	  /* string token out of range */
      return emptyString;
    }

}

#if 0
#include "base/crit.h"
#include "base/systhr.h"
static int Initialized = 0;

#ifdef XP_UNIX

#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <unistd.h>

union _semarg
{
	int					val;
	struct semid_ds		*buf;
	ushort				*array;
};

int		_si18n_semMutex;
pid_t	_lockOwner = 0;
int		_si18n_waitQueue;
struct	sembuf	_semop_p[1] = {0,-1,0};
struct	sembuf	_semop_v[1] = {0,1,0};


void _si18n_lock(MutexPointer p)
{
	pid_t	current_pid = getpid();

	semop(*((int*)p),_semop_p,1);
	if (current_pid != _lockOwner)
	{
		semop(*((int*)p),_semop_v,1);
		semop(_si18n_waitQueue,_semop_p,1);	/* wait for access */
		semop(*((int*)p),_semop_p,1);
		_lockOwner = current_pid;
	}

	semop(*((int*)p),_semop_v,1);

}

void _si18n_unlock(MutexPointer p)
{
	semop(*((int*)p),_semop_p,1);

	_lockOwner = 0;
	semop(_si18n_waitQueue,_semop_v,1);		/* signal one waiter */

	semop(*((int*)p),_semop_v,1);

}

MutexImplementation	_nlsMutexImplementation =
{
	&_si18n_semMutex,
	_si18n_lock,
	_si18n_unlock
};

MutexImplementation*	_pMutexImplementation = &_nlsMutexImplementation;


void InitSynchronization(void)
{
	union _semarg	val;

	val.val = 1;			/* to avoid an aligment fault on certain platforms */

	_si18n_semMutex  = semget(IPC_PRIVATE,1,IPC_CREAT | 0600);
	_si18n_waitQueue = semget(IPC_PRIVATE,1,IPC_CREAT | 0600);

	semctl(_si18n_semMutex,0,SETVAL,val);
	semctl(_si18n_waitQueue,0,SETVAL,val);
}

#else
#include "lock.h"
#include <windows.h>

static CRITICAL_SECTION	_csMutex;

void _lock(MutexPointer p)
{
	EnterCriticalSection((CRITICAL_SECTION*)p);
}

void _unlock(MutexPointer p)
{
	LeaveCriticalSection((CRITICAL_SECTION*)p);
}

MutexImplementation	_nlsMutexImplementation =
{
	&_csMutex,
	_lock,
	_unlock
};

MutexImplementation*	_pMutexImplementation = &_nlsMutexImplementation;

void InitSynchronization(void)
{
	InitializeCriticalSection(&_csMutex);
}

#endif

#endif

