/** BEGIN COPYRIGHT BLOCK
 * Copyright (C) 2001 Sun Microsystems, Inc.  Used by permission.
 * Copyright (C) 2005 Red Hat, Inc.
 * All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * END COPYRIGHT BLOCK **/
#ifndef RESHASH_H
#define RESHASH_H
/**********************************************************************
    Hash  --> Tree --> ValueList      

    ValueList: language per item, each list associated with one key
    Tree:  contains multiple keys
    Hash:  Based on hash to decide withc tree to use for lookup

***********************************************************************/

/* 
  Valuelist, each item contains 
    language:  ISO two or four letters 
    value:     UTF-8 encoding strings
 */
typedef struct ValueNode {
	char *language;
	char *value;
    struct ValueNode *next;
} ValueNode;


/*  
  Current: BINARY TREE
  Future: balanced tree for high search performance
 */
typedef struct TreeNodeStruct {
	ValueNode *vlist;
    char *key;
	char *value;
    struct TreeNodeStruct *left;
    struct TreeNodeStruct *right;
} TreeNode;


typedef struct ResHash {
    char *name;  /* name of hash table */
    TreeNode *treelist;
} ResHash;


ResHash * ResHashCreate(char * name);
int ResHashAdd(ResHash *res, char *key, char *value, char *language);
const char *ResHashSearch(ResHash *res, char *key, char *language);
void ResHashDestroy(ResHash *res);

#endif

